<?php
/**
 * List of Admin Filter Hooks.
 *
 * @package   Give-Square
 * @copyright Copyright (c) 2018, GiveWP
 * @license   https://opensource.org/licenses/gpl-license GNU Public License
 * @since     1.0.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Register Section for Gateway Settings.
 *
 * @param array $sections List of sections.
 *
 * @since 1.0.0
 *
 * @return mixed
 */
function give_square_register_sections( $sections ) {

	$sections['square-settings'] = __( 'Square', 'give-square' );

	return $sections;
}

add_filter( 'give_get_sections_gateways', 'give_square_register_sections' );

/**
 * Add "Square" advanced settings.
 *
 * @param array $section List of sections.
 *
 * @since 1.0.0
 *
 * @return mixed
 */
function give_square_connect_add_advanced_section( $section ) {
	$section['square'] = __( 'Square', 'give-square' );

	return $section;
}

add_filter( 'give_get_sections_advanced', 'give_square_connect_add_advanced_section' );

/**
 * Add advanced Square settings.
 *
 * New tab under Settings > Advanced that allows users to use their own API key.
 *
 * @param array $settings List of settings.
 *
 * @since 1.0.0
 *
 * @return mixed
 */
function give_square_register_advanced_settings( $settings ) {

	$current_section = give_get_current_setting_section();

	// Bailout, if not a square section under advanced tab.
	if ( 'square' !== $current_section ) {
		return $settings;
	}

	// Disconnect Square OAuth API when manual API keys are enabled.
	if ( give_square_is_manual_api_keys_enabled() ) {
		give_square_disconnect_oauth();
	}

	switch ( $current_section ) {
		case 'square':
			$settings = array(
				array(
					'id'   => 'give_title_square_advanced',
					'type' => 'title',
				),
				array(
					'name'    => __( 'Square API Keys', 'give-square' ),
					'desc'    => give_square_is_manual_api_keys_enabled() ? sprintf(
						__( '<div style="padding: 8px 10px; background:#FFF; margin: 8px 0 0; border-radius: 4px;"><strong>Square API keys enabled:</strong> Please enter your API keys on the payment gateways settings screen. <a href="%1$s">Click here</a> to configure your API keys.</div>', 'give-square' ),
						esc_url_raw( admin_url() . 'edit.php?post_type=give_forms&page=give-settings&tab=gateways&section=square-settings' )
					) : __( 'Enable if you would like to use your own API keys instead of Square Connect via oAuth API.', 'give-square' ),
					'id'      => 'square_api_keys',
					'type'    => 'radio_inline',
					'default' => 'disabled',
					'options' => array(
						'enabled'  => __( 'Enabled', 'give-square' ),
						'disabled' => __( 'Disabled', 'give-square' ),
					),
				),
				array(
					'name'          => __( 'Square Styles', 'give-square' ),
					'desc'          => sprintf(
						'%1$s <a href="%2$s" target="_blank">%3$s</a>.',
						__( 'Edit the properties above to match the look and feel of your WordPress theme. These styles will be applied to the Square Credit Card field only. Any valid CSS property can be defined, however, it must be formatted as JSON, not CSS. For more information on Styling Square CC field please see the square documentation: ', 'give-square' ),
						esc_url_raw( 'https://docs.givewp.com/square-style' ),
						__( 'see the Square developer docs', 'give-square' )
					),
					'wrapper_class' => 'give-square-styles-wrap',
					'id'            => 'square_styles',
					'type'          => 'textarea',
					'default'       => '{}',
				),
				array(
					'id'   => 'give_title_square_advanced',
					'type' => 'sectionend',
				)
			);

			break;
	} // End switch().


	// Output.
	return $settings;

}

add_filter( 'give_get_settings_advanced', 'give_square_register_advanced_settings', 10, 1 );

/**
 * This function is used to add a new cron interval
 *
 * @param array $schedules Cron Interval Schedules.
 *
 * @since 1.0.0
 *
 * @return mixed
 */
function give_square_add_monthly_cron_interval( $schedules ) {
	$schedules['monthly'] = array(
		'interval' => 2592000,
		'display'  => esc_html__( 'Monthly', 'give-square' ),
	);

	return $schedules;
}

add_filter( 'cron_schedules', 'give_square_add_monthly_cron_interval' );

