<?php

namespace GiveSquare\GatewayConnect\Actions;

use Give\Framework\Exceptions\Primitives\Exception;
use Give\Framework\PaymentGateways\Log\PaymentGatewayLog;
use GiveSquare\GatewayConnect\Api\CreateWebhookSubscription;
use GiveSquare\GatewayConnect\Modals\SquareAccount;
use GiveSquare\Square\Api\Exceptions\ApiRequestException;
use GiveSquare\Square\Api\Traits\ApiRequestHelpers;

/**
 * @see https://developer.squareup.com/docs/webhooks/webhook-subscriptions-api
 *
 * @since 2.0.0
 */
class CreateSquareWebhook
{
    use ApiRequestHelpers;

    /**
     * @since 2.0.0
     */
    public function __invoke(string $slug, string $mode): bool
    {
        $idempotencyKey = uniqid();

        $subscription = [
            "name" => "GiveWP",
            "notification_url" => site_url($slug),
            "event_types" => [
                "invoice.updated",
                "invoice.payment_made",
                "subscription.created",
                "subscription.updated",
            ],
        ];

        $requestData = [
            "subscription" => $subscription,
            "idempotency_key" => $idempotencyKey,
        ];

        try {
            $squareAccount = SquareAccount::make($mode);
            $personalAccessToken = $squareAccount->getPersonalAccessToken();
            $webhook = (new CreateWebhookSubscription())($requestData, $mode, $personalAccessToken);

            if ($webhook->enabled) {
                $squareAccount->updateWebhook($webhook);

                return $webhook->enabled;
            }

            return false;
        } catch (Exception $e) {
            $errorMessage = $e->getMessage();

            if ($e instanceof ApiRequestException) {
                $errorMessage = $e->getSquareErrorMessage();
            }

            PaymentGatewayLog::error(
                sprintf(__('[Square] Fail to setup %s webhook.', 'give-square'),
                    $mode),
                [
                    'Message' => $errorMessage,
                ]
            );

            return false;
        }
    }
}
