<?php

namespace GiveSquare\GatewayConnect\Api;

use Give\Framework\Exceptions\Primitives\InvalidArgumentException;
use GiveSquare\GatewayConnect\Exceptions\AccountNotFoundException;
use GiveSquare\GatewayConnect\Exceptions\RetrieveTokenException;
use GiveSquare\GatewayConnect\Modals\SquareAccount;
use stdClass;
use WP_Error;

/**
 * @since 1.2.0
 */
class RetrieveAccessToken
{
    /**
     * @since 2.0.0 Fix: correctly check 'error_renew_token_failed' return
     * @since      1.2.0
     * @throws AccountNotFoundException|RetrieveTokenException
     */
    public function __invoke(SquareAccount $squareAccount): stdClass
    {
        if (!$squareAccount->isConnected()) {
            throw new InvalidArgumentException('Unable to retrieve access token for disconnected Square account');
        }

        $response = wp_remote_get(
            give_square_connect_server_url(
                sprintf('/square/connect_%3$s.php?action=renew&refresh_token=%1$s&connect_id=%2$s',
                    $squareAccount->getRefreshToken(), $squareAccount->getConnectionId(), $squareAccount->getMode()
                )
            )
        );

        $this->validateWpErrorInResponse($response);

        $responseBody = wp_remote_retrieve_body($response);

        if ('error_renew_token_failed' === $responseBody) {
            throw new AccountNotFoundException(
                esc_html__(
                    sprintf(
                        'Unable to renew the access token. Please reconnect with your Square account and then try again. Error:  %1$s',
                        $response['response']['code'] . ' - ' . $response['response']['message']
                    )
                )
            );
        }

        return json_decode($responseBody);
    }

    /**
     * @param array|WP_Error $response
     *
     * @throws RetrieveTokenException
     */
    private function validateWpErrorInResponse($response)
    {
        if (is_wp_error($response)) {
            throw new RetrieveTokenException(
                esc_html__(
                    sprintf(
                        'Retrieve square access token api request failed. Error:  %1$s',
                        $response->get_error_message()
                    )
                )
            );
        }
    }
}
