<?php

namespace GiveSquare\GatewayConnect\Modals;

use Give\Framework\Exceptions\Primitives\Exception;
use Give\Framework\Exceptions\Primitives\InvalidArgumentException;
use Give\Helpers\Call;
use GiveSquare\GatewayConnect\Actions\RetrieveSquareTokenScopes;
use GiveSquare\GatewayConnect\Api\RetrieveAccessToken;
use stdClass;

/**
 * @since 1.2.0
 */
class SquareAccount
{
    /**
     * @var string
     */
    private $mode;

    /**
     * @since 1.2.0
     */
    public function __construct()
    {
        $this->mode = give_is_test_mode() ? 'test' : 'live';
    }

    /**
     * @since 1.2.0
     */
    public static function make(string $mode): self
    {
        $self = new static();
        $self->mode = $mode;

        $self->validateMode();

        return $self;
    }

    /**
     * @since 1.2.0
     */
    public function getRefreshToken(): string
    {
        $optionName = $this->isTestMode() ?
            'give_square_sandbox_refresh_token' :
            'give_square_live_refresh_token';

        return give_get_option($optionName, '');
    }

    /**
     * @since 1.2.0
     */
    public function getConnectionId(): string
    {
        $optionName = $this->isTestMode() ?
            'give_square_sandbox_connect_id' :
            'give_square_live_connect_id';

        return give_get_option($optionName, '');
    }

    /**
     * @since 1.2.0
     */
    public function getLocationId(): string
    {
        $optionName = $this->isTestMode() ?
            'give_square_sandbox_location_id' :
            'give_square_live_location_id';

        return give_get_option($optionName, '');
    }

    /**
     * @since 1.2.0
     */
    public function getMode(): string
    {
        return $this->mode;
    }

    /**
     * @since 1.2.0
     */
    public function isConnected(): bool
    {
        if ( ! function_exists('give_square_is_connected')) {
            require_once GIVE_SQUARE_PLUGIN_DIR . '/includes/helpers.php';
        }

        return $this->isTestMode() ?
            give_square_sandbox_is_connected() || give_square_is_manual_api_keys_enabled() :
            give_square_is_connected() || give_square_is_manual_api_keys_enabled();
    }

    /**
     * @since 2.0.0
     * @throws Exception
     */
    public function validateScopes(): bool
    {
        $expectedScopes = [
            'PAYMENTS_WRITE',
            'PAYMENTS_READ',
            'PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS',
            'MERCHANT_PROFILE_READ',
            'CUSTOMERS_READ',
            'CUSTOMERS_WRITE',
            'ITEMS_READ',
            'ITEMS_WRITE',
            'SUBSCRIPTIONS_READ',
            'SUBSCRIPTIONS_WRITE',
            'ORDERS_READ',
            'ORDERS_WRITE',
            'INVOICES_READ',
            'INVOICES_WRITE',
        ];

        $currentScopes = $this->getTokenScopes();

        if (array_diff($expectedScopes, $currentScopes)) {
            $currentScopes = (new RetrieveSquareTokenScopes())();
            $this->updateTokenScopes($currentScopes);
        }

        $result = array_diff($expectedScopes, $currentScopes);

        return ! $result;
    }

    /**
     * @since 2.0.1 Ensure it always returns an array
     * @since      2.0.0
     */
    public function getTokenScopes(): array
    {
        $currentScopes = $this->isTestMode() ?
            give_get_option('give_square_sandbox_access_token_scopes') :
            give_get_option('give_square_live_access_token_scopes');

        return is_array($currentScopes) ? $currentScopes : [];
    }

    /**
     * @since 2.0.0
     */
    private function updateTokenScopes(array $scopes)
    {
        $optionName = $this->isTestMode() ?
            'give_square_sandbox_access_token_scopes' :
            'give_square_live_access_token_scopes';

        give_update_option($optionName, $scopes);
    }

    /**
     * @since 2.0.0
     */
    public function isPersonalAccessTokenSetup(): bool
    {
        return (bool)$this->getPersonalAccessToken();
    }

    /**
     * @since 2.0.0
     */
    public function getPersonalAccessToken()
    {
        return $this->isTestMode() ?
            give_get_option('give_square_sandbox_personal_access_token') :
            give_get_option('give_square_live_personal_access_token');
    }

    /**
     * @since 2.0.0
     */
    public function isWebhookSetup(): bool
    {
        return (bool)$this->getWebhook();
    }

    /**
     * @since 2.0.0
     */
    public function getWebhook()
    {
        return $this->isTestMode() ?
            give_get_option('give_square_sandbox_webhook') :
            give_get_option('give_square_live_webhook');
    }

    /**
     * @since 2.0.0
     */
    public function updateWebhook(stdClass $webhook)
    {
        $optionName = $this->isTestMode() ?
            'give_square_sandbox_webhook' :
            'give_square_live_webhook';

        give_update_option($optionName, $webhook);
    }

    /**
     * @since 1.2.0
     * Returns date in "Y-m-d H:i:s" format
     */
    public function getAccessTokenRenewDate(): string
    {
        return $this->isTestMode() ?
            give_get_option('give_square_renew_token_sandbox_trigger_date') :
            give_get_option('give_square_renew_token_live_trigger_date');
    }

    /**
     * @since 1.2.0
     */
    public function getAccountType(bool $formatted = false): string
    {
        $accountType = $this->isTestMode() ?
            'sandbox' :
            'live';

        if ($formatted) {
            $accountType = $this->isTestMode() ?
                esc_html__('Sandbox', 'give-square') :
                esc_html__('Live', 'give-square');
        }

        return $accountType;
    }

    /**
     * @since 1.2.0
     */
    public function disconnectAccount()
    {
        if ($this->isTestMode()) {
            give_delete_option('give_square_sandbox_access_token');
            //give_delete_option('give_square_sandbox_personal_access_token'); //Necessary to create webhooks
            give_delete_option('give_square_sandbox_webhook');
            give_delete_option('give_square_sandbox_access_token_scopes');
            give_delete_option('give_square_sandbox_application_id');
            give_delete_option('give_square_sandbox_business_location');
            give_delete_option('give_square_sandbox_expires_at');
            give_delete_option('give_square_sandbox_merchant_id');
            give_delete_option('give_square_sandbox_token_type');
            give_delete_option('give_square_sandbox_refresh_token');
            give_delete_option('give_square_sandbox_connect_id');
            give_delete_option('give_square_sandbox_is_connected');

            // Delete Locations List Cache when Square is disconnected.
            \Give_Cache::delete('give_cache_square_sandbox_locations_list');
        } else {
            give_delete_option('give_square_live_access_token');
            //give_delete_option('give_square_live_personal_access_token'); //Necessary to create webhooks
            give_delete_option('give_square_live_webhook');
            give_delete_option('give_square_live_access_token_scopes');
            give_delete_option('give_square_live_application_id');
            give_delete_option('give_square_business_location');
            give_delete_option('give_square_live_expires_at');
            give_delete_option('give_square_live_merchant_id');
            give_delete_option('give_square_live_token_type');
            give_delete_option('give_square_live_refresh_token');
            give_delete_option('give_square_live_connect_id');
            give_delete_option('give_square_is_connected');

            // Delete Locations List Cache when Square is disconnected.
            \Give_Cache::delete('give_cache_square_locations_list');
        }
    }

    /**
     * Returns date in "Y-m-d H:i:s" format
     * @since 1.2.0
     */
    private function updateAccessTokenRenewDate(): bool
    {
        $optionName = $this->isTestMode() ?
            'give_square_renew_token_sandbox_trigger_date' :
            'give_square_renew_token_live_trigger_date';

        return give_update_option($optionName, current_time('mysql'));
    }

    /**
     * @since 1.2.0
     */
    public function renewAccessToken()
    {
        $accessToken = Call::invoke(RetrieveAccessToken::class, $this);
        $encryptedAccessToken = give_square_encrypt_string($accessToken->access_token);

        if ($this->isTestMode()) {
            give_update_option('give_square_sandbox_access_token', $encryptedAccessToken);
            give_update_option('give_square_sandbox_expires_at', $accessToken->expires_at);
        } else {
            give_update_option('give_square_live_access_token', $encryptedAccessToken);
            give_update_option('give_square_live_expires_at', $accessToken->expires_at);
        }

        $this->updateAccessTokenRenewDate();
    }

    /**
     * @since 1.2.0
     */
    private function validateMode()
    {
        if (!in_array($this->mode, ['test', 'live'])) {
            throw new InvalidArgumentException('Invalid mode. Allowed values are test and live.');
        }
    }

    /**
     * @since 1.2.0
     * @return void
     */
    private function isTestMode(): bool
    {
        return 'test' === $this->mode;
    }
}
