<?php

namespace GiveSquare\GatewayConnect\Webhooks;

use Give\Framework\Exceptions\Primitives\Exception;
use Give\Framework\PaymentGateways\Log\PaymentGatewayLog;
use GiveSquare\GatewayConnect\Actions\CreateSquareWebhook;
use GiveSquare\GatewayConnect\Modals\SquareAccount;
use stdClass;
use WP;

/**
 * @since 2.0.0
 */
class SetupWebhooks
{
    /**
     * @var string
     */
    private $slug = 'give-square-webhooks-listener';

    /**
     * @since 2.0.0
     */
    public function eventListener(WP $query)
    {
        if ($this->slug !== $query->request) {
            return;
        }

        // Retrieve the request's body and parse it as JSON.
        $body = @file_get_contents('php://input');
        $eventJson = json_decode($body);

        if ( ! $eventJson ||
             ! isset($eventJson->type) ||
             ! isset($eventJson->data->id) ||
             ! isset($eventJson->data->object)
        ) {
            status_header(500);

            PaymentGatewayLog::error(
                '[Square] An error occurred while processing a webhook.',
                [
                    'Event Type' => $eventJson->type ?? 'null',
                    'Event Data ID' => $eventJson->data->id ?? 'null',
                    'Event Data Object' => $eventJson->data->object ?? 'null',
                    'Event Json' => $eventJson,
                ]
            );

            die('-1'); // Failed
        }

        status_header(200);

        PaymentGatewayLog::debug(
            sprintf('[Square Webhooks] Trigger %s event for %s id %s.',
                $eventJson->type,
                $eventJson->data->type,
                $eventJson->data->id),
            [
                '$eventJson' => $eventJson,
            ]
        );

        /**
         * Fire the action
         *
         * @since 2.0.0
         *
         * @param stdClass $eventJson
         */
        do_action("givewp_square_webhook_event", $eventJson);

        die('1'); // Completed successfully
    }

    /**
     * AJAX check webhooks.
     *
     * @since 2.0.0
     * @throws Exception
     */
    public function ajaxCheckSquareWebhooks()
    {
        $squareLiveAccount = SquareAccount::make('live');
        $squareSandboxAccount = SquareAccount::make('test');

        $data = [
            'live_webhooks_setup' => false,
            'sandbox_webhooks_setup' => false,
        ];

        if ( ! $squareLiveAccount->isPersonalAccessTokenSetup()) {
            give_delete_option('give_square_live_webhook');
            $data['live_webhooks_setup'] = 'unconfigured';
        } elseif ($squareLiveAccount->isWebhookSetup()) {
            $data['live_webhooks_setup'] = true;
        } elseif ( ! $squareLiveAccount->isWebhookSetup()) {
            $liveWebhook = (new CreateSquareWebhook())($this->slug, 'live');
            $data['live_webhooks_setup'] = $liveWebhook ? true : 'error';
        }

        if ( ! $squareSandboxAccount->isPersonalAccessTokenSetup()) {
            give_delete_option('give_square_sandbox_webhook');
            $data['sandbox_webhooks_setup'] = 'unconfigured';
        } elseif ($squareSandboxAccount->isWebhookSetup()) {
            $data['sandbox_webhooks_setup'] = true;
        } elseif ( ! $squareSandboxAccount->isWebhookSetup()) {
            $sandboxWebhook = (new CreateSquareWebhook())($this->slug, 'test');
            $data['sandbox_webhooks_setup'] = $sandboxWebhook ? true : 'error';
        }

        wp_send_json_success($data);

        wp_die();
    }

    /**
     * Force recheck of webhooks.
     *
     * @since 2.0.0
     * @throws Exception
     */
    public function ajaxForceRecheckWebhooks()
    {
        give_delete_option('give_square_live_webhook');
        give_delete_option('give_square_sandbox_webhook');

        $this->ajaxCheckSquareWebhooks();
    }
}
