<?php

namespace GiveSquare\PaymentGateway\Actions;

use Give\Donations\Models\Donation;
use Give\Donations\Models\DonationNote;
use Give\Framework\Exceptions\Primitives\Exception;
use Give\Framework\PaymentGateways\Log\PaymentGatewayLog;
use GiveSquare\Square\Api\Exceptions\ApiRequestException;
use Square\Apis\CustomersApi;
use Square\Exceptions\ApiException;
use Square\Http\ApiResponse;
use Square\Models\Address;
use Square\Models\CreateCustomerRequest;

/**
 * @since 1.2.0
 */
class GetSquareCustomerId
{
    /** @var CustomersApi $customerApi */
    public $customerApi;

    /**
     * @since 2.0.0 Use new square/square SDK to obtain the $customerId
     * @since      1.2.0
     *
     * @throws ApiException|Exception
     */
    public function __invoke(Donation $donation): string
    {
        $apiClient = give_square_get_api_client();
        $this->customerApi = $apiClient->getCustomersApi();

        $customerId = $this->getSquareCustomerId($donation);

        if ($customerId) {
            give_update_meta($donation->id, '_give_square_donation_customer_id', $customerId);
            give()->donations->notes->insert(
                new DonationNote(
                    [
                        'donationId' => $donation->id,
                        'content' => "Customer ID: {$customerId}",
                    ]
                )
            );
        }

        return $customerId;
    }

    /**
     * @since 2.0.0
     *
     * @throws ApiRequestException|ApiException
     */
    private function getSquareCustomerId(Donation $donation): string
    {
        $existingCustomerId = give_get_meta($donation->donorId, '_give_square_donor_customer_id', true);

        if ($existingCustomerId) {
            $apiResponse = $this->customerApi->retrieveCustomer($existingCustomerId);
            if ($apiResponse->isError()) {
                $apiResponse = $this->createSquareCustomer($donation);
                $existingCustomerId = '';
            }
        } else {
            $apiResponse = $this->createSquareCustomer($donation);
        }

        if ($apiResponse->isSuccess() && $apiResponse->getResult()->getCustomer()) {
            $customerId = $apiResponse->getResult()->getCustomer()->getId();
            if (empty($existingCustomerId)) {
                give_update_meta($donation->donorId, '_give_square_donor_customer_id', $customerId);
            }

            return $customerId;
        }

        if ($apiResponse->isError()) {
            PaymentGatewayLog::error(
                sprintf('[Square] Unable to %s customer details from Square for donation %s.',
                    $existingCustomerId ? 'retrieve' : 'create', $donation->id),
                [
                    'Payment Gateway' => $donation->gateway()->getId(),
                    'Donation' => $donation->id,
                    'Errors' => $apiResponse->getErrors(),
                ]
            );

            $body = json_decode($apiResponse->getBody());

            if (property_exists($body, 'errors')) {
                throw new ApiRequestException($body);
            }
        }

        return '';
    }

    /**
     * @since 2.0.0
     *
     * @throws ApiException
     */
    private function createSquareCustomer(Donation $donation): ApiResponse
    {
        $customer = new CreateCustomerRequest();
        $customer->setGivenName($donation->firstName);
        $customer->setFamilyName($donation->lastName);
        $customer->setEmailAddress($donation->email);

        if (give_is_setting_enabled(give_get_option('give_square_collect_billing_details', false))) {
            $billingAddress = new Address();
            $billingAddress->setAddressLine1($donation->billingAddress->address1);

            if ( ! empty($donation->billingAddress->address2)) {
                $billingAddress->setAddressLine2($donation->billingAddress->address2);
            }

            $billingAddress->setLocality($donation->billingAddress->city);
            $billingAddress->setAdministrativeDistrictLevel1($donation->billingAddress->state);
            $billingAddress->setCountry($donation->billingAddress->country);

            if ( ! empty($donation->billingAddress->zip)) {
                $billingAddress->setPostalCode($donation->billingAddress->zip);
            }

            $billingAddress->setFirstName($donation->firstName);
            $billingAddress->setLastName($donation->lastName);

            $customer->setAddress($billingAddress);
        }

        return $this->customerApi->createCustomer($customer);
    }
}
