<?php

namespace GiveSquare\PaymentGateway;

use Exception;
use Give\Donations\Models\Donation;
use Give\Donations\Models\DonationNote;
use Give\Donations\ValueObjects\DonationStatus;
use Give\Framework\PaymentGateways\Commands\GatewayCommand;
use Give\Framework\PaymentGateways\Commands\PaymentComplete;
use Give\Framework\PaymentGateways\Exceptions\PaymentGatewayException;
use Give\Framework\PaymentGateways\PaymentGateway;
use Give\Helpers\Call;
use GiveSquare\PaymentGateway\Actions\CreateSquarePayment;
use GiveSquare\Square\Api\Exceptions\ApiRequestException;
use GiveSquare\Square\Api\RefundPayment;

/**
 * @since 1.2.0
 */
class CreditCardGateway extends PaymentGateway
{
    /**
     * @since 1.2.0
     */
    public function getLegacyFormFieldMarkup(int $formId, array $args): string
    {
        return give_square_credit_card_form($formId, $args, false);
    }

    /**
     * @since 1.2.0
     */
    public static function id(): string
    {
        return 'square';
    }

    /**
     * @since 1.2.0
     */
    public function getId(): string
    {
        return self::id();
    }

    /**
     * @since 1.2.0
     */
    public function getName(): string
    {
        return esc_html__('Square Gateway', 'give-square');
    }

    /**
     * @since 1.2.0
     */
    public function getPaymentMethodLabel(): string
    {
        return esc_html__('Credit Card', 'give-square');
    }

    /**
     * @since 3.0.0
     */
    public function formSettings(int $formId): array
    {
        return [
            'applicationID' => give_square_get_application_id(),
            'locationID' => give_square_get_location_id(),
        ];
    }

    /**
     * @since 3.0.0
     */
    public function enqueueScript(int $formId)
    {
        $assets = $this->getScriptAsset(trailingslashit(GIVE_SQUARE_PLUGIN_DIR) . '/build/creditCardGateway.asset.php');

        wp_enqueue_script(
            self::id(),
            trailingslashit(GIVE_SQUARE_PLUGIN_URL) . 'build/creditCardGateway.js',
            $assets['dependencies'],
            $assets['version'],
            true
        );
    }

    /**
     * @since 1.3.0 Access square payment method from gateway data
     * @since 1.2.1 Make code compatible with Give 2.21.0
     * @since 1.2.0
     *
     * @throws PaymentGatewayException
     * @throws \Give\Framework\Exceptions\Primitives\Exception
     */
    public function createPayment(Donation $donation, $gatewayData): GatewayCommand
    {
        try {
            $sourceId = $gatewayData['squarePaymentMethodId'];
            $transactionId = (new CreateSquarePayment())($donation, $sourceId);

            return new PaymentComplete($transactionId);
        } catch (Exception $e) {
            $donation->status = DonationStatus::FAILED();
            $errorMessage = $e->getMessage();

            if ($e instanceof ApiRequestException) {
                $errorMessage = $e->getSquareErrorMessage();

                if ($e->getSquarePaymentId()) {
                    $donation->gatewayTransactionId = $e->getSquarePaymentId();
                }
            }

            $donation->save();

            DonationNote::create([
                'donationId' => $donation->id,
                'content' => sprintf(esc_html__('Donation failed. Reason: %s', 'give-square'), $errorMessage)
            ]);

            throw new PaymentGatewayException($errorMessage);
        }
    }

    /**
     * @since 1.2.0
     */
    public function refundDonation(Donation $donation)
    {
        // Backward compatibility.
        // I find out that we do not need to create unique identifier in donation for  the Square idempotency key.
        // We can utilize donation purchase key.
        $idempotencyKey = give_get_meta($donation->id, '_give_square_donation_idempotency_key', true) ?:
            $donation->purchaseKey;

        $refundRequestArgs = [
            'idempotency_key' => $idempotencyKey,
            'payment_id' => $donation->gatewayTransactionId,
            'amount_money' => [
                'amount' => absint($donation->amount->formatToMinorAmount()),
                'currency' => $donation->amount->getCurrency()->getCode()
            ],
            'reason' => sprintf(esc_html__('Refund from GiveWP Admin on %s', 'give-square'), home_url())

        ];

        return Call::invoke(RefundPayment::class, $refundRequestArgs)->id;
    }

    /**
     * @see https://developer.wordpress.org/block-editor/reference-guides/packages/packages-dependency-extraction-webpack-plugin/#wordpress
     *
     * @since 3.0.0
     */
    protected function getScriptAsset(string $path): array
    {
        return file_exists($path) ? require $path : ['dependencies' => [], 'version' => filemtime($path)];
    }
}
