import { Gateway } from "./Interfaces";
import { CreditCard, PaymentForm } from "react-square-web-payments-sdk";
import { __, sprintf } from "@wordpress/i18n";

type SquareGatewaySettings = {
    applicationID: string;
    locationID: string;
};
let settings: SquareGatewaySettings; // These data are sent by the backend.

/**
 * Payment tokenization is the process of taking sensitive payment information and replacing it with a nonsensitive
 * generated number called a token. This var stores this value, and then we send it to the backend to process the
 * payment without the original credit card data being transferred through the network.
 *
 * @see https://developer.squareup.com/blog/online-payments-with-square-and-react/
 */
let squareCardNonce = ""; // We are hoisting it to use it inside the CreditCardFields() and beforeCreatePayment() functions

const creditCardGateway: Gateway = {
    id: "square",
    initialize() {
        settings = this.settings; // We are hoisting it to use it inside the Fields() function
    },
    beforeCreatePayment: async function (values: FormData): Promise<object> {
        const squarePayButton: HTMLElement =
            document.querySelector("#rswp-card-button");

        squarePayButton.click();

        while (squareCardNonce === "") {
            let squareCardError = document.querySelector(
                ".sq-card-message-error"
            );
            if (squareCardError) {
                throw new Error(
                    __(
                        sprintf(
                            "Square gateway error: %s",
                            squareCardError.innerHTML
                        ),
                        "give-square"
                    )
                );
            }
            await new Promise((resolve) => setTimeout(resolve, 100));
        }

        return {
            "square-card-nonce": squareCardNonce,
        };
    },
    Fields() {
        // @ts-ignore
        const { useWatch } = window.givewp.form.hooks;
        const gatewayId = useWatch({ name: "gatewayId" });
        const isCurrentGatewayActive = gatewayId === creditCardGateway.id;

        if (!isCurrentGatewayActive) {
            return;
        }

        return <CreditCardFields />;
    },
};

const CreditCardFields = () => {
    return (
        <>
            <PaymentForm
                applicationId={settings.applicationID}
                cardTokenizeResponseReceived={(token, verifiedBuyer) => {
                    squareCardNonce = token ? token.token : "";
                }}
                locationId={settings.locationID}
            >
                <CreditCard
                    buttonProps={{
                        css: {
                            display: "none",
                        },
                    }}
                />
            </PaymentForm>
        </>
    );
};

// @ts-ignore
window.givewp.gateways.register(creditCardGateway);
